# Tour Migrator

This gem started from a bunch of ruby and bash scripts to help
with migrating a KRPano tour to our portal's format and to clean and check a
SVG floorplan.

In late 2018 this turned into a gem and the code made reusable, also providing
interfaces in classes/modules, to allow reusing programmaticly.
The code is used for the

- [tour upload feature](https://cloud.factro.com/?p=task&pi=882113a3-1f6d-4537-aa2e-d1597d91b3db), and for the
- [floorplan upload feature](https://cloud.factro.com/Project/adae9b92-22e1-4207-840f-e4b4db9efed8/psb/882113a3-1f6d-4537-aa2e-d1597d91b3db?p=task&pi=a7edaa8f-5044-4546-b924-4aa661cb50f0).

## Release a new version

1. increase the version in `lib/tour_migrator/version.rb`. Please obey SemVer for version numbers.
2. `bundle`
3. `rake build`
4. add changes & commit all changes with commit message: "chore: release $VERSION"
5. `git tag $VERSION`
6. `git push --tags`
7. upload the created file at https://gems.intocities.com

## How to migrate a complete tour manually

1. get the tour's zip file from the NAS (customer folder)
2. unpack the zip
3. `migrate_tour customer/ c105xx` (example: `migrate_tour REWE_Anika_Scherf c10531`)
4. go to [the spreadsheet](https://docs.google.com/spreadsheets/d/1Sg0c-k0YBYWy-cV9-SsJHbrzsKXXCwKV0mGn3kRzuiE/edit?ts=5be2eda7#gid=0) and export the scenes and titles as csv (menu: File -> Download as -> Comma seperated values (csv))
5. use the csv to set the scene titles in the xml:
  example: `tag_scenes ~/Downloads/Tourszenenbenennung.csv scenes.xml`
6. write output to `scenes.xml`
7. upload the folder to `server2`, `/var/www/cdn.intocities.com/virtual_tours/tour_files`

## Usage

### Easier usage tip

**TODO:** remove this 'easier usage tip' when this is an installable gem.

Checkout this repo and setup aliases  to run these scripts comfortably.
To setup aliases, open your `~/.profile` and paste at then end:

```sh
alias floorplan='ruby ~/versioned/durchdiestadt/tour_migrator/exe/floorplan'
alias floorplan_svgo='svgo --config=/Users/gil/versioned/durchdiestadt/tour_migrator/support/svgo.config.yaml'
alias migrate_tour='~/versioned/durchdiestadt/tour_migrator/exe/migrate_tour'
alias tag_scenes='ruby ~/versioned/durchdiestadt/tour_migrator/exe/tag_scenes'
```

## Provided Executables

Run an executable with: `bundle exec tour_migrator $name_of_the_executable`

### migrate_tour

Migrate Tours to the format the new portal expects.

Please check the checklist it prints at the end.

### tag_scenes

Takes a CSV file and the tour's `scenes.xml` and automatically writes the given tags.
The CSV must contain at least these columns:

`scene, title`

The columns `lat, lng` are optional.

### floorplan

Fixes the floorplans which Adobe Illustrator exports.

Tries to find

- levels
- level
- hotspots
- hotspots

and sets the classes.

Also reports statistics about the svg file.

### Minify floorplan with `svgo`

Use the supplied config to minify the floorplan with [`svgo`](https://github.com/svg/svgo).

`svgo --config=floorplan/svgo.config.yaml floorplan.svg`

## Debugging

### XML Errors

In case you run into an XML reading error, try `xmllint`
(linux: install `libxml2-utils`, macos: already installed)
to find the error. Probably an unescaped ampersand or so.
