module TourMigrator
  module Floorplan
    class Cleaner
      attr_reader :svg
      def initialize(svg)
        @svg = svg
      end

      def perform
        @svg = remove_comments
        @svg = remove_unneeded_ids
        @svg
      end

      private

      def remove_comments
        svg.xpath('//comment()').each(&:unlink)
        svg
      end

      def remove_unneeded_ids
        ids_and_elements = svg.css('[id]').group_by do |element|
          element['id']
        end

        unneeded_ids = ids_and_elements.keys - needed_ids

        unneeded_ids.each do |unneeded_id|
          ids_and_elements[unneeded_id].map do |element|
            element.remove_attribute('id')
          end
        end

        svg
      end

      def needed_ids
        @needed_ids ||= [].tap do |needed_ids|
          needed_ids << svg.css('.hotspot[id], .level[id], clipPath[id], .linktotour[id]').map { |element| element['id'] }
          needed_ids << svg.css('[for]').map { |element| element['for'] }

          svg.css('use').each do |element|
            # href contains a hash as first char, since it's a reference
            needed_ids << element['xlink:href'][1..-1] if element['xlink:href']
          end
        end.flatten.uniq
      end
    end
  end
end
