module TourMigrator
  module Floorplan
    class IllustratorExportFixer
      attr_reader :svg
      def initialize(svg)
        @svg = svg
      end

      def perform
        svg.css('[id^="linktotour"]').each do |hotspot|
          hotspot['class'] = 'linktotour'
          hotspot['id'] = encoding_fix(hotspot['id'].to_s).gsub('linktotour_', '')
        end

        hotspots = svg.css('[id*="scene"]')
        hotspots.each do |hotspot|
          hotspot['class'] = ('hotspot ' + hotspot['class'].to_s).split.uniq.join(' ')

          # some illustrator versions add _2_ to the end of ids, when the last character is not a space.
          hotspot['id'] = hotspot['id'].gsub('_2_', '') if hotspot['id'].to_s.end_with?('_2_')

          hotspot['id'] = encoding_fix(hotspot['id'].to_s)

          # some illustrator versions add the real name of an element to a data-attribute
          # which is not ascii-encoded...
          if !hotspot['id'].to_s.include?('/') && hotspot['data-name']
            hotspot['id'] = hotspot['data-name']
            hotspot.remove_attribute('data-name')
          end
        end

        svg.css('[id*="switch_"]').each_with_index do |level_switch, index|
          level_switch['class'] = 'level-switch'
          level_switch['href'] = encoding_fix(level_switch['id'].to_s)
                                 .gsub(/level.switch/i, '').tr('_', ' ').strip
          level_switch['id'] = 'level-switch-' + index.to_s
        end

        levels = svg.css('[id^="level"], [id^="Level"]')
        levels.each do |level|
          next if level['id'].include?('switch')

          level['class'] = ('level ' + level['class'].to_s).split.uniq.join(' ')
          level['id'] = encoding_fix(level['id'].to_s.gsub(/level/i, ''))
        end

        %w[hotspots levels].each do |class_name|
          svg.css('[id*="' + class_name + '"]').each do |element|
            element['class'] = (class_name + ' ' + element['class'].to_s).split.uniq.join(' ')
            element.remove_attribute('id')
          end
        end

        svg
      end

      private

      def encoding_fix(string)
        string = string.gsub('_x2F_', '/')
                       .gsub('_x5F_', '_')
                       .gsub('x2F_', '_')
                       .gsub('_x2D_', '')
                       .split('_').uniq.compact.join('_')

        string = string.slice(1, string.chars.count) if string.index('_').zero?

        string = string[0...-2] if string[-2..-1] == '_1'

        string
      end
    end
  end
end
