require 'nokogiri'

module TourMigrator
  class GenerateScenes
    attr_accessor :todos
    attr_reader :customer_id

    def initialize(file, customer_id)
      raise ArgumentError, 'file must be file!' unless File.file?(file)
      raise ArgumentError, 'customer_id must be a string' unless customer_id.is_a?(String)

      @file = file
      @customer_id = customer_id.split('/').last

      @todos = []
    end

    def doc
      @doc ||= Nokogiri::XML(@file) do |config|
        config.strict
        config.noblanks
      end
    end

    def perform
      return @perform if defined?(@perform)

      new_doc = doc.css('krpano').clone

      new_doc = cleanup(new_doc)
      new_doc = scenes(new_doc)
      new_doc = hotspots(new_doc)

      todos << 'add exittocitytour hotspot!' unless new_doc.css('hotspot[exittocitytour]').count.positive?

      @perform = new_doc
    end

    private

    def cleanup(new_doc)
      # 1. remove all child elements of <krpano> which are not a <scene>
      new_doc.css('krpano > *:not(scene)').map(&:unlink)

      # 2. remove all comments
      new_doc.xpath('//comment()').remove

      # 3. clean krpano element
      krpano = new_doc.css('krpano:first')
      krpano.remove_attr('onstart')

      new_doc
    end

    def scenes(new_doc)
      scenes = new_doc.css('scene')
      scenes.each do |scene|
        # 4.1 remove attributes
        scene.remove_attribute('onstart')
        scene.remove_attribute('urlname')
        scene.remove_attribute('thumburl')

        # 4.4 check if images folder is same name as scene name
        scene.css('image *[url], preview[url]').each do |url_element|
          next if url_element[:url].start_with?(scene[:name])

          file_parts = url_element[:url].split('/')

          if file_parts[0] != 'assets' && !file_parts.include?('%s')
            todos << "mv -v '#{file_parts[0..-2].join('/')}' '#{scene[:name]}';"
          end

          file_parts_without_tiles = file_parts[1..-1].keep_if { |part| !part.include?('.tiles') }

          url_element[:url] = [scene[:name], file_parts_without_tiles].join('/')
        end

        # 4.2 prefix scene name with customer id if not present
        if scene[:name] && !scene[:name].to_s.start_with?(customer_id)
          scene[:name] = "#{customer_id}/#{scene[:name]}"
        end

        # 4.3 remove <action name="updateMap">
        scene.css('action[name="updateMap"]').map(&:unlink)
      end

      new_doc
    end

    def hotspots(new_doc)
      new_doc.css('hotspot').each do |hotspot|
        # 5.0 remove pad-hotspot
        if hotspot[:name] == 'pad'
          hotspot.unlink
          next
        end

        if hotspot[:name].start_with?('detailfoto')
          hotspot[:style] = 'details'
          hotspot[:name] = hotspot[:pic]

          %w[onover onout onclick url pic].each do |removal_attribute|
            hotspot.remove_attribute(removal_attribute) if hotspot[removal_attribute]
          end
        else
          # 5.1 change style
          hotspot[:style] = 'vTourHotspotStyle'

          # 5.2 rewrite linkedscene
          if hotspot[:linkedscene] && !hotspot[:linkedscene].to_s.start_with?(customer_id)
            hotspot[:linkedscene] = customer_id + "/#{hotspot[:linkedscene]}"
          end

          todos.add("check linkedscene of `#{hotspot[:name]}`!") if hotspot[:linkedscene]&.to_s&.end_with?('000')

          # Remove onclick handler
          next unless hotspot[:onclick]

          hotspot.remove_attribute('onclick')

          hotspot[:exittocitytour] = true
        end
      end

      new_doc
    end
  end
end
