module TourMigrator
  class SceneTagger
    # Creates new instance.
    # @param [Nokogiri::XML::Document] xml
    # @param [Hash] scene_mappings With String keys
    def initialize(xml, scene_mappings)
      raise ArgumentError('must give all params') unless xml || scene_mappings
      raise ArgumentError if scene_mappings&.keys&.count&.to_i.zero?

      @xml = xml
      @scene_mappings = scene_mappings
    end

    # Tags the scenes from xml
    # @return [Nokogiri::XML::Document]
    def perform
      scene_mappings.each_pair do |scene_name, attributes|
        next if scene_name.to_s == ''

        scenes = xml.css("scene[name$=#{scene_name}]")

        next unless scenes

        scene = scenes&.first

        next unless scene
        next unless attributes

        scene[:title] = attributes['title']
        scene[:lat] = attributes['lat'].to_s.tr(',', '.').to_f if attributes['lat'].to_s != ''
        scene[:lng] = attributes['lng'].to_s.tr(',', '.').to_f if attributes['lng'].to_s != ''
      end

      xml
    end

    private

    attr_reader :xml, :scene_mappings
  end
end
